from __future__ import annotations

from mixam_sdk.item_specification.enums.lamination import Lamination
from mixam_sdk.item_specification.enums.pre_drilled_holes import PreDrilledHoles
from mixam_sdk.item_specification.interfaces.component_protocol import (
    TwoSidedComponent as ITwoSidedComponent,
    LaminatedComponent as ILaminatedComponent,
    FoiledComponent as IFoiledComponent,
)
from mixam_sdk.item_specification.models.bound_component import BoundComponent
from mixam_sdk.item_specification.models.component_support import ComponentSupport
from mixam_sdk.item_specification.models.item_specification import ItemSpecification
from mixam_sdk.metadata.product.models.product_metadata import ProductMetadata
from mixam_sdk.metadata.product.services.validation_result import ValidationResult
from .base import DefaultComponentValidator


class PrimaryComponentValidator(DefaultComponentValidator):
    """
    Validator that applies rules that are only relevant to the primary component.
    This moves checks that reference the "primaryComponent" from the Java version
    out of the base validator so non-primary components do not get these errors.

    Primary-component rules implemented here:
    - Custom size support (and range check if metadata present in future)
    - Substrate combination validation against product.substrate_types
    - Lamination support against substrate type and weight
    - Orientation enforcement for auto-orientable products
    - Pre-drilled holes availability/options (non-bound only)
    - Foiling dependencies (requires lamination etc.) including back-side for two-sided
    """

    def validate(self, product_metadata: ProductMetadata, item_specification: ItemSpecification, component: ComponentSupport, result: ValidationResult, base_path: str) -> None:
        # Run generic rules that are safe for all components (format, substrate design)
        super().validate(product_metadata, item_specification, component, result, base_path)

        # Primary-only: Validate colours vs product colours metadata (front)
        try:
            allowed_colours = {o.colours.name for o in product_metadata.colours_metadata.colours_options}
            if allowed_colours and component.colours.name not in allowed_colours:
                result.add_error(
                    path=f"{base_path}.colours",
                    message=f"Colours '{component.colours.name}' not available for this product.",
                    code="colours.unavailable",
                    allowed=sorted(list(allowed_colours)),
                )
        except Exception:
            # Ignore metadata errors
            pass

        # Primary-only: Validate back colours for two-sided components
        try:
            if isinstance(component, ITwoSidedComponent):
                allowed_back_colours = {o.colours.name for o in product_metadata.colours_metadata.back_colours_options}
                if allowed_back_colours and component.back_colours.name not in allowed_back_colours:
                    result.add_error(
                        path=f"{base_path}.backColours",
                        message=f"Unsupported back colour '{component.back_colours.name}'.",
                        code="back_colours.unavailable",
                        allowed=sorted(list(allowed_back_colours)),
                    )
        except Exception:
            pass

        # Custom size support: only primary component should be checked strictly
        try:
            if component.has_custom_size() and product_metadata.custom_size_metadata is None:
                result.add_error(
                    path=f"{base_path}.customSize",
                    message="Custom size not supported by this product.",
                    code="custom_size.unsupported",
                )
        except Exception:
            pass

        # Validate substrate combination (type -> colour -> weight) by IDs using primary substrate_types
        try:
            substrate_type_metadata = next((t for t in product_metadata.substrate_types if t.id == component.substrate.type_id), None)
            valid_combo = False
            if substrate_type_metadata is not None:
                substrate_colour_metadata = next((c for c in substrate_type_metadata.substrate_colours if c.id == component.substrate.colour_id), None)
                if substrate_colour_metadata is not None:
                    substrate_weight_metadata = next((w for w in substrate_colour_metadata.weights if w.id == component.substrate.weight_id), None)
                    valid_combo = substrate_weight_metadata is not None
            if not valid_combo:
                result.add_error(
                    path=f"{base_path}.substrate",
                    message=(
                        f"Invalid substrate combination. Type ID: {component.substrate.type_id}, "
                        f"Colour ID: {component.substrate.colour_id}, Weight ID: {component.substrate.weight_id}"
                    ),
                    code="substrate.combo.invalid",
                )
        except Exception:
            pass

        # Orientation rule: auto-orientable products enforce default orientation only (primary only)
        try:
            if product_metadata.auto_orientable and product_metadata.default_orientation is not None:
                if component.orientation != product_metadata.default_orientation:
                    result.add_error(
                        path=f"{base_path}.orientation",
                        message=(
                            f"This product is auto orientable and only supports {product_metadata.default_orientation.name} orientation. "
                            "(Note: Both portrait & landscape artwork files are auto detected and processed accordingly)"
                        ),
                        code="orientation.not_supported",
                        required=product_metadata.default_orientation.name,
                    )
        except Exception:
            pass

        # Pre-drilled holes: only allowed when options exist and not for bound components' special case
        try:
            if not isinstance(component, BoundComponent):
                options = product_metadata.pre_drilled_holes_metadata.pre_drilled_hole_options
                if not options and component.pre_drilled_holes != PreDrilledHoles.NONE:
                    result.add_error(
                        path=f"{base_path}.preDrilledHoles",
                        message="Pre drilled holes are not supported by this product.",
                        code="pre_drilled_holes.unsupported",
                    )
                elif component.pre_drilled_holes != PreDrilledHoles.NONE:
                    allowed = {o.pre_drilled_holes.name for o in options}
                    if component.pre_drilled_holes.name not in allowed:
                        result.add_error(
                            path=f"{base_path}.preDrilledHoles",
                            message="Unsupported Pre Drilled Hole option.",
                            code="pre_drilled_holes.option_invalid",
                            allowed=sorted(list(allowed)),
                        )
        except Exception:
            pass

        # Lamination validations for laminated primary components
        try:
            if isinstance(component, ILaminatedComponent) and component.lamination != Lamination.NONE:
                substrate_type_metadata = next((t for t in product_metadata.substrate_types if t.id == component.substrate.type_id), None)
                if substrate_type_metadata is None or not substrate_type_metadata.allow_lamination:
                    result.add_error(
                        path=f"{base_path}.lamination",
                        message="Unsupported lamination. Specified substrate type does not support lamination.",
                        code="lamination.substrate_type_unsupported",
                    )
                else:
                    try:
                        substrate_colour_metadata = next((c for c in substrate_type_metadata.substrate_colours if c.id == component.substrate.colour_id), None)
                        substrate_weight_metadata = next((w for w in substrate_colour_metadata.weights if w.id == component.substrate.weight_id), None) if substrate_colour_metadata else None
                        if substrate_weight_metadata is not None and not substrate_weight_metadata.supports_lamination:
                            result.add_error(
                                path=f"{base_path}.lamination",
                                message="Lamination is not supported for this substrate weight.",
                                code="lamination.substrate_weight_unsupported",
                            )
                    except Exception:
                        pass
        except Exception:
            pass

        # Foiling validations for primary (front + back if two-sided)
        try:
            if isinstance(component, IFoiledComponent):
                front_has = component.foiling.has_foiling()
                if front_has:
                    if not product_metadata.foiling_metadata.front_foiling:
                        result.add_error(
                            path=f"{base_path}.foiling",
                            message="Foiling is not supported by this product.",
                            code="foiling.unsupported",
                        )
                    else:
                        if isinstance(component, ILaminatedComponent):
                            lam = component.lamination
                            if lam != Lamination.NONE:
                                ok = any(lam in opt.supported_laminations for opt in product_metadata.foiling_metadata.front_foiling)
                                if not ok:
                                    result.add_error(
                                        path=f"{base_path}.foiling",
                                        message="Unsupported foiling option for selected lamination.",
                                        code="foiling.lamination_incompatible",
                                    )
                            else:
                                result.add_error(
                                    path=f"{base_path}.foiling",
                                    message="Unsupported foiling option. Foiling is only available for Laminated Components.",
                                    code="foiling.requires_lamination",
                                )
            if isinstance(component, ITwoSidedComponent):
                # Back lamination rules (SAME_AS_FRONT and availability)
                try:
                    back_lam = component.back_lamination
                    lamination_metadata = product_metadata.lamination_metadata
                    back_lamination_options = lamination_metadata.back_options if lamination_metadata is not None else []
                    # If no back options configured and a back lamination is set, it's unsupported
                    if not back_lamination_options and back_lam != Lamination.NONE:
                        result.add_error(
                            path=f"{base_path}.backLamination",
                            message="This product does not support back lamination.",
                            code="back_lamination.unsupported",
                        )
                    else:
                        # SAME_AS_FRONT behavior if listed among back options
                        same_as_front_available = any(
                            str(opt.lamination).upper() == "SAME_AS_FRONT"
                            for opt in back_lamination_options
                        )
                        if same_as_front_available:
                            # When SAME_AS_FRONT is available, enforce equality if a back lamination is chosen (non-NONE)
                            if back_lam != Lamination.NONE and isinstance(component, ILaminatedComponent) and component.lamination != back_lam:
                                result.add_error(
                                    path=f"{base_path}.backLamination",
                                    message="Back lamination has to be same as front lamination or none.",
                                    code="back_lamination.same_as_front_required",
                                )
                        else:
                            # If SAME_AS_FRONT is not available: back lamination, when set, must be among back options by value
                            if back_lam != Lamination.NONE:
                                allowed_values = {int(opt.value) for opt in back_lamination_options}
                                if allowed_values and back_lam.get_value() not in allowed_values and back_lam.value not in allowed_values:
                                    # Our Lamination enum has get_value(); try both .value and .get_value()
                                    result.add_error(
                                        path=f"{base_path}.backLamination",
                                        message="Unsupported back lamination. This product accepts configured back options only.",
                                        code="back_lamination.option_invalid",
                                        allowed=sorted(list(allowed_values)),
                                    )
                except Exception:
                    pass

                # Back foiling rules
                back_has = component.back_foiling.has_foiling()
                if back_has:
                    if not product_metadata.foiling_metadata.back_foiling:
                        result.add_error(
                            path=f"{base_path}.backFoiling",
                            message="Foiling is not supported by this product on the back side.",
                            code="back_foiling.unsupported",
                        )
                    else:
                        lam = component.back_lamination
                        if lam != Lamination.NONE:
                            ok = any(lam in opt.supported_laminations for opt in product_metadata.foiling_metadata.back_foiling)
                            if not ok:
                                result.add_error(
                                    path=f"{base_path}.backFoiling",
                                    message="Unsupported back foiling option for selected back lamination.",
                                    code="back_foiling.lamination_incompatible",
                                )
                        else:
                            result.add_error(
                                path=f"{base_path}.backFoiling",
                                message="Unsupported foiling option. Back foiling is only available with back lamination.",
                                code="back_foiling.requires_lamination",
                            )
        except Exception:
            pass


__all__ = ["PrimaryComponentValidator"]



from __future__ import annotations

from pydantic import BaseModel, Field, ConfigDict

from mixam_sdk.metadata.product.models.simple_fold_option import SimpleFoldOption


class FoldingOptions(BaseModel):

    portrait_options: list[SimpleFoldOption] = Field(
        default_factory=list,
        alias="portraitOptions",
    )

    landscape_options: list[SimpleFoldOption] = Field(
        default_factory=list,
        alias="landscapeOptions",
    )

    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
        frozen=True,
        strict=True,
        validate_assignment=True,
    )


__all__ = ["FoldingOptions"]



from __future__ import annotations

from enum import Enum


class SantaType(Enum):
    QUOTE = "QUOTE"
    PUBLICATION = "PUBLICATION"
    PRINT_ON_DEMAND = "PRINT_ON_DEMAND"
    FOURTHWALL = "FOURTHWALL"
    ADOBE_EXPRESS = "ADOBE_EXPRESS"


__all__ = ["SantaType"]



from __future__ import annotations

from dataclasses import dataclass
from typing import Dict, Tuple

from mixam_sdk.item_specification.enums.orientation import Orientation
from mixam_sdk.item_specification.enums.unit_format import UnitFormat
from mixam_sdk.item_specification.enums.simple_fold import SimpleFold


# Constants
MM_IN_INCH = 25.4
MM_IN_POINT = 2.8346456693
MM_IN_300_DPI_PIXEL = 11.811023622
INCH_IN_POINT = 72.0
INCH_IN_300_DPI_PIXEL = 300.0

PAGE_SIZE_UNKNOWN = -1
PAGE_SIZE_SPINE = -99
PAGE_SIZE_JACKET = -98
PAGE_SIZE_TOLERANCE = 20  # percent
SECONDARY_SIZE_TOLERANCE = 10.0  # mm
PAGE_HEIGHT_TOLERANCE = 40  # percent
INACCURACY = 0.5
FLAPS_SIZE = 80
NARROW_FLAPS_SIZE = 40
GLUE_TRAP_SIZE = 7
HEIGHT_ADDITION_SIZE = 3
DUST_JACKET_THRESHOLD = 2.0
DEFAULT_BLEED = 3.0

# DIN A sizes (mm) A0..A8
DIN_SIZES: Tuple[Tuple[int, int], ...] = (
    (841, 1189),  # A0
    (594, 841),   # A1
    (420, 594),   # A2
    (297, 420),   # A3
    (210, 297),   # A4
    (148, 210),   # A5
    (105, 148),   # A6
    (74, 105),    # A7
    (52, 74),     # A8
)

# B sizes (mm) B0..B9
B_SIZES: Tuple[Tuple[int, int], ...] = (
    (1000, 1414),
    (707, 1000),
    (500, 707),
    (353, 500),
    (250, 353),
    (176, 250),
    (125, 176),
    (88, 125),
    (62, 88),
    (44, 62),
)


@dataclass(frozen=True)
class SecondarySizeInfo:
    id: int
    name: str
    width: float
    height: float
    imperial_width: float | None = None
    imperial_height: float | None = None

    def get_dims(self, unit: UnitFormat) -> Tuple[float, float]:
        if unit == UnitFormat.METRIC:
            return float(self.width), float(self.height)
        if self.imperial_width is not None and self.imperial_height is not None:
            return float(self.imperial_width), float(self.imperial_height)
        return mm_to_inch(self.width), mm_to_inch(self.height)


_secondary_info_map: Dict[int, SecondarySizeInfo] = {
    1: SecondarySizeInfo(1, "Novel (127mm x 203mm)", 127, 203, 5, 8),
    2: SecondarySizeInfo(2, "Standard (132mm x 197mm)", 132, 197, 5.2, 7.76),
    3: SecondarySizeInfo(3, "Demy (138mm x 216mm)", 138, 216, 5.43, 8.5),
    4: SecondarySizeInfo(4, "US Royal (152mm x 229mm)", 152, 229, 6, 8.98),
    5: SecondarySizeInfo(5, "Royal (156mm x 234mm)", 156, 234, 6.14, 9.21),
    6: SecondarySizeInfo(6, "210mm Square", 210, 210, 8.27, 8.27),
    7: SecondarySizeInfo(7, "Large Format Portrait (210mm x 280mm)", 210, 280, 8.27, 11),
    8: SecondarySizeInfo(8, "Dl Small 100", 100, 210, 3.94, 8.27),
    9: SecondarySizeInfo(9, "DL 105mm x 210mm", 105, 210, 4.13, 8.27),
    10: SecondarySizeInfo(10, "A5 Long (105mm x 297mm)", 105, 297, 4.13, 11.69),
    11: SecondarySizeInfo(11, "148mm Square", 148, 148, 5.82, 5.82),
    12: SecondarySizeInfo(12, "105mm Square", 105, 105, 4.13, 4.13),
    13: SecondarySizeInfo(13, "120mm Square", 120, 120),
    14: SecondarySizeInfo(14, "A1 Long", 420, 1188),
    15: SecondarySizeInfo(15, "A2 Long", 297, 840),
    16: SecondarySizeInfo(16, "DL Small", 99, 210),
    17: SecondarySizeInfo(17, "B1", 707, 1000),
    18: SecondarySizeInfo(18, "B2", 500, 707),
    19: SecondarySizeInfo(19, "183mm x 273mm", 183, 273),
    20: SecondarySizeInfo(20, "150mm x 265mm", 150, 265),
    21: SecondarySizeInfo(21, "100mm Square", 100, 100),
    22: SecondarySizeInfo(22, "55mm x 85mm", 55, 85),
    23: SecondarySizeInfo(23, "DL", 110, 220),
    24: SecondarySizeInfo(24, "C5", 162, 229),
    25: SecondarySizeInfo(25, "C4", 229, 324),
    26: SecondarySizeInfo(26, "C6", 162, 114),
    27: SecondarySizeInfo(27, "Square 155", 155, 155),
    28: SecondarySizeInfo(28, "Square 170", 170, 170),
    29: SecondarySizeInfo(29, "DL large", 229, 114),
    30: SecondarySizeInfo(30, "50mm x 90mm", 50, 90),
    31: SecondarySizeInfo(31, "54mm x 86mm", 54, 86),
    32: SecondarySizeInfo(32, "55mm Square", 55, 55),
    33: SecondarySizeInfo(33, "B5", 175, 245),
    34: SecondarySizeInfo(34, "B4", 248, 346),
    35: SecondarySizeInfo(35, "98mm Square", 98, 98),
    36: SecondarySizeInfo(36, "A6 Long", 74, 210),
    37: SecondarySizeInfo(37, "A7 Long", 52, 148),
    38: SecondarySizeInfo(38, "B0", 1000, 1414),
    39: SecondarySizeInfo(39, "170mm x 590mm", 170, 590),
    40: SecondarySizeInfo(40, "Large format", 1185, 1750),
    41: SecondarySizeInfo(41, "A7 mini", 74, 98),
    42: SecondarySizeInfo(42, "Large Format Trade (170mm x 240mm)", 170, 240),
    43: SecondarySizeInfo(43, "508mm x 762mm", 508, 762, 20, 30),
    44: SecondarySizeInfo(44, "762mm x 1016mm", 762, 1016, 30, 40),
    45: SecondarySizeInfo(45, "1016mm x 1524mm", 1016, 1524, 40, 60),
    46: SecondarySizeInfo(46, "5.5\" x 8.5\"", 140, 216, 5.5, 8.5),
    47: SecondarySizeInfo(47, "8.5\" x 11\"", 216, 279, 8.5, 11),
    48: SecondarySizeInfo(48, "8.5\" x 22\"", 216, 559, 8.5, 22),
    49: SecondarySizeInfo(49, "4.25\" x 5.5\"", 108, 140, 4.25, 5.5),
    50: SecondarySizeInfo(50, "4\" x 6\"", 102, 152, 4, 6),
    51: SecondarySizeInfo(51, "5\" x 7\"", 127, 178, 5, 7),
    52: SecondarySizeInfo(52, "6\" x 9\"", 152, 229, 6, 9),
    53: SecondarySizeInfo(53, "8\" x 10\"", 203, 254, 8, 10),
    54: SecondarySizeInfo(54, "6\" x 11\"", 152, 279, 6, 11),
    55: SecondarySizeInfo(55, "11\" x 17\"", 279, 432, 11, 17),
    56: SecondarySizeInfo(56, "6\" x 6\"", 152, 152, 6, 6),
    57: SecondarySizeInfo(57, "12\" x 12\"", 305, 305, 12, 12),
    58: SecondarySizeInfo(58, "4.75\" x 4.75\"", 121, 121, 4.75, 4.75),
    59: SecondarySizeInfo(59, "12\" x 18\"", 305, 457, 12, 18),
    60: SecondarySizeInfo(60, "13\" x 19\"", 330, 483, 13, 19),
    61: SecondarySizeInfo(61, "18\" x 24\"", 457, 610, 18, 24),
    62: SecondarySizeInfo(62, "19\" x 27\"", 483, 686, 19, 27),
    63: SecondarySizeInfo(63, "24\" x 36\"", 610, 914, 24, 36),
    64: SecondarySizeInfo(64, "26\" x 39\"", 660, 991, 26, 39),
    65: SecondarySizeInfo(65, "8.5\" x 14\"", 216, 356, 8.5, 14),
    66: SecondarySizeInfo(66, "UK Standard", 157, 240, 6.18, 9.45),
    67: SecondarySizeInfo(67, "US Standard", 170, 260),
    68: SecondarySizeInfo(68, "Manga Standard", 127, 191),
    69: SecondarySizeInfo(69, "5\" x 8\"", 127, 203, 5, 8),
    70: SecondarySizeInfo(70, "5.06\" x 7.81\"", 129, 198, 5.06, 7.81),
    71: SecondarySizeInfo(71, "5.25\" x 8\"", 133, 203, 5.25, 8),
    72: SecondarySizeInfo(72, "6.14\" x 9.21\"", 156, 234, 6.14, 9.21),
    73: SecondarySizeInfo(73, "6.69\" x 9.61\"", 170, 244, 6.69, 9.61),
    74: SecondarySizeInfo(74, "7\" x 10\"", 178, 254, 7, 10),
    75: SecondarySizeInfo(75, "7.44\" x 9.69\"", 189, 246, 7.44, 9.69),
    76: SecondarySizeInfo(76, "7.5\" x 9.25\"", 191, 235, 7.5, 9.25),
    77: SecondarySizeInfo(77, "6\" x 8.25\"", 152, 210, 6, 8.25),
    78: SecondarySizeInfo(78, "8.25\" Square", 210, 210, 8.25, 8.25),
    79: SecondarySizeInfo(79, "8.5\" Square", 216, 216, 8.5, 8.5),
    80: SecondarySizeInfo(80, "7\" Square", 178, 178, 7, 7),
    81: SecondarySizeInfo(81, "140mm Square", 140, 140),
    82: SecondarySizeInfo(82, "B6", 125, 176),
    83: SecondarySizeInfo(83, "US Trade", 152, 229, 6, 9),
    84: SecondarySizeInfo(84, "Business card", 51, 89),
    85: SecondarySizeInfo(85, "Postcard", 89, 140),
    86: SecondarySizeInfo(86, "Postcard", 108, 152),
    87: SecondarySizeInfo(87, "8\" x 11\"", 203, 279, 8, 11),
    88: SecondarySizeInfo(88, "11\" x 14\"", 279, 356, 11, 14),
    89: SecondarySizeInfo(89, "8\" x Square", 203, 203, 8, 8),
    90: SecondarySizeInfo(90, "B format UK", 129, 198),
    91: SecondarySizeInfo(91, "Pinched Crown Quarto", 171, 246, 6.73, 9.65),
    92: SecondarySizeInfo(92, "Crown Quarto", 189, 246, 7.44, 9.68),
    93: SecondarySizeInfo(93, "280 Square", 280, 280),
    94: SecondarySizeInfo(94, "7.25\" x 9.5\"", 184, 241, 7.25, 9.5),
    95: SecondarySizeInfo(95, "Pocket Book 4.25\" x 6.87\"", 108, 174, 4.25, 6.87),
    96: SecondarySizeInfo(96, "Small Square 7.5\"", 191, 191, 7.5, 7.5),
    97: SecondarySizeInfo(97, "Quarto 9.5\" x 12\"", 241, 305, 9.5, 12),
    98: SecondarySizeInfo(98, "7\" x 9\"", 178, 229, 7, 9),
    99: SecondarySizeInfo(99, "200mm x 200mm", 200, 200),
    100: SecondarySizeInfo(100, "200mm x 250mm", 200, 250),
    101: SecondarySizeInfo(101, "200mm x 300mm", 200, 300),
    102: SecondarySizeInfo(102, "200mm x 600mm", 200, 600),
    103: SecondarySizeInfo(103, "300mm x 300mm", 300, 300),
    104: SecondarySizeInfo(104, "300mm x 400mm", 300, 400),
    105: SecondarySizeInfo(105, "300mm x 600mm", 300, 600),
    106: SecondarySizeInfo(106, "300mm x 1000mm", 300, 1000),
    107: SecondarySizeInfo(107, "400mm x 400mm", 400, 400),
    108: SecondarySizeInfo(108, "400mm x 500mm", 400, 500),
    109: SecondarySizeInfo(109, "400mm x 600mm", 400, 600),
    110: SecondarySizeInfo(110, "500mm x 500mm", 500, 500),
    111: SecondarySizeInfo(111, "500mm x 600mm", 500, 600),
    112: SecondarySizeInfo(112, "500mm x 700mm", 500, 700),
    113: SecondarySizeInfo(113, "500mm x 750mm", 500, 750),
    114: SecondarySizeInfo(114, "500mm x 1000mm", 500, 1000),
    115: SecondarySizeInfo(115, "600mm x 600mm", 600, 600),
    116: SecondarySizeInfo(116, "600mm x 750mm", 600, 750),
    117: SecondarySizeInfo(117, "600mm x 800mm", 600, 800),
    118: SecondarySizeInfo(118, "750mm x 1000mm", 750, 1000),
    119: SecondarySizeInfo(119, "10\" x Square", 254, 254, 10, 10),
    120: SecondarySizeInfo(120, "11.8\" x Square", 300, 300, 11.8, 11.8),
    121: SecondarySizeInfo(121, "3.5\" x 8.5\"", 89, 216, 3.5, 8.5),
    122: SecondarySizeInfo(122, "4.25\" x 11\"", 108, 280, 4.25, 11),
    123: SecondarySizeInfo(123, "7\" x 8.5\"", 178, 216, 7, 8.5),
    124: SecondarySizeInfo(124, "7.5\" x 8.5\"", 191, 216, 7.5, 8.5),
    125: SecondarySizeInfo(125, "9\" x 12\"", 229, 305, 9, 12),
    126: SecondarySizeInfo(126, "9\" x 16\"", 229, 406, 9, 16),
    127: SecondarySizeInfo(127, "8.5\" x 17\"", 216, 432, 8.5, 17),
    128: SecondarySizeInfo(128, "11.5\" x 17.5\"", 292, 445, 11.5, 17.5),
    129: SecondarySizeInfo(129, "17\" x 22\"", 432, 559, 17, 22),
    130: SecondarySizeInfo(130, "11\" x 25.5\"", 279, 648, 11, 25.5),
    131: SecondarySizeInfo(131, "8\" x 12\"", 203, 305, 8, 12),
    132: SecondarySizeInfo(132, "10\" x 15\"", 254, 381, 10, 15),
    133: SecondarySizeInfo(133, "5.5\" x 8.5\"", 140, 216, 5.5, 8.5),
    134: SecondarySizeInfo(134, "3.5\" x 8.5\"", 89, 216, 3.5, 8.5),
    135: SecondarySizeInfo(135, "4\" x 9\"", 102, 229, 4, 9),
    136: SecondarySizeInfo(136, "2\" x 8\"", 51, 203, 2, 8),
    137: SecondarySizeInfo(137, "3.66\" x 4.25\"", 93, 108, 3.66, 4.25),
    138: SecondarySizeInfo(138, "3\" x 4\"", 76, 102, 3, 4),
    139: SecondarySizeInfo(139, "4\" x 4\"", 102, 102, 4, 4),
    140: SecondarySizeInfo(140, "5.8\" x 9.25\"", 147, 235, 5.8, 9.25),
    141: SecondarySizeInfo(141, "8.375\" x 10.875\"", 213, 276, 8.375, 10.875),
    142: SecondarySizeInfo(142, "5.375\" x 8.375\"", 137, 213, 5.375, 8.375),
    143: SecondarySizeInfo(143, "5.25\" x 8.375\"", 133, 213, 5.25, 8.375),
    144: SecondarySizeInfo(144, "10mm x 10mm", 10, 10),
    145: SecondarySizeInfo(145, "15mm x 15mm", 15, 15),
    146: SecondarySizeInfo(146, "20mm x 20mm", 20, 20),
    147: SecondarySizeInfo(147, "21mm x 21mm", 21, 21),
    148: SecondarySizeInfo(148, "25mm x 25mm", 25, 25),
    149: SecondarySizeInfo(149, "10mm x 30mm", 10, 30),
    150: SecondarySizeInfo(150, "30mm x 30mm", 30, 30),
    151: SecondarySizeInfo(151, "15mm x 35mm", 15, 35),
    152: SecondarySizeInfo(152, "35mm x 35mm", 35, 35),
    153: SecondarySizeInfo(153, "35mm x 105mm", 35, 105),
    154: SecondarySizeInfo(154, "35mm x 210mm", 35, 210),
    155: SecondarySizeInfo(155, "35mm x 316mm", 35, 316),
    156: SecondarySizeInfo(156, "20mm x 40mm", 20, 40),
    157: SecondarySizeInfo(157, "40mm x 40mm", 40, 40),
    158: SecondarySizeInfo(158, "15mm x 45mm", 15, 45),
    159: SecondarySizeInfo(159, "45mm x 45mm", 45, 45),
    160: SecondarySizeInfo(160, "48mm x 70mm", 48, 70),
    161: SecondarySizeInfo(161, "20mm x 50mm", 20, 50),
    162: SecondarySizeInfo(162, "25mm x 50mm", 25, 50),
    163: SecondarySizeInfo(163, "40mm x 50mm", 40, 50),
    164: SecondarySizeInfo(164, "50mm x 50mm", 50, 50),
    165: SecondarySizeInfo(165, "51mm x 298mm", 51, 298),
    166: SecondarySizeInfo(166, "51mm x 420mm", 51, 420),
    167: SecondarySizeInfo(167, "55mm x 85mm", 55, 85),
    168: SecondarySizeInfo(168, "60mm x 60mm", 60, 60),
    169: SecondarySizeInfo(169, "21mm x 68mm", 21, 68),
    170: SecondarySizeInfo(170, "70mm x 70mm", 70, 70),
    171: SecondarySizeInfo(171, "71mm x 96mm", 71, 96),
    172: SecondarySizeInfo(172, "80mm x 80mm", 80, 80),
    173: SecondarySizeInfo(173, "45mm x 95mm", 45, 95),
    174: SecondarySizeInfo(174, "95mm x 95mm", 95, 95),
    175: SecondarySizeInfo(175, "95mm x 145mm", 95, 145),
    176: SecondarySizeInfo(176, "98mm x 420mm", 98, 420),
    177: SecondarySizeInfo(177, "120mm x 125mm", 120, 125),
    178: SecondarySizeInfo(178, "140mm x 297mm", 140, 297),
    179: SecondarySizeInfo(179, "124mm x 140mm", 124, 140),
    180: SecondarySizeInfo(180, "98mm x 210mm", 98, 210),
    181: SecondarySizeInfo(181, "Silver Age (6.875\" x 10.25\")", 175, 260, 6.875, 10.25),
    182: SecondarySizeInfo(182, "Golden Age (7.375\" x 10.25\")", 187, 260, 7.375, 10.25),
    183: SecondarySizeInfo(183, "8.75\" x 11\"", 222, 279, 8.75, 11),
    184: SecondarySizeInfo(184, "6.63\" x 10.25\"", 168, 260, 6.63, 10.25),
    185: SecondarySizeInfo(185, "5.125\" x 7\"", 130, 178, 5.125, 7),
    186: SecondarySizeInfo(186, "4.125\" x 5.875\"", 105, 149, 4.125, 5.875),
    187: SecondarySizeInfo(187, "895mm x 1280mm", 895, 1280),
    188: SecondarySizeInfo(188, "700mm x 1000mm", 700, 1000),
    189: SecondarySizeInfo(189, "1000mm x 1400mm", 1000, 1400),
    190: SecondarySizeInfo(190, "130mm x 150mm", 130, 150),
    191: SecondarySizeInfo(191, "138mm x 297mm", 138, 297),
    192: SecondarySizeInfo(192, "30mm x 70mm", 30, 70),
    193: SecondarySizeInfo(193, "199mm x 210mm", 199, 210),
    194: SecondarySizeInfo(194, "130mm x 190mm", 130, 190),
    195: SecondarySizeInfo(195, "297mm Square", 297, 297),
    196: SecondarySizeInfo(196, "240mm x 340mm", 240, 340),
    197: SecondarySizeInfo(197, "300mm x 800mm", 300, 800),
    198: SecondarySizeInfo(198, "400mm x 1500mm", 400, 1500),
    199: SecondarySizeInfo(199, "750mm x 750mm", 750, 750),
    200: SecondarySizeInfo(200, "1000mm x 1000mm", 1000, 1000),
    201: SecondarySizeInfo(201, "1000mm x 1500mm", 1000, 1500),
    202: SecondarySizeInfo(202, "1000mm x 2000mm", 1000, 2000),
    203: SecondarySizeInfo(203, "800mm x 1200mm", 800, 1200),
    204: SecondarySizeInfo(204, "1000mm x 1200mm", 1000, 1200),
    205: SecondarySizeInfo(205, "2\" x 3.5\"", 51, 89, 2, 3.5),
    206: SecondarySizeInfo(206, "2\" x 6\"", 51, 152, 2, 6),
    207: SecondarySizeInfo(207, "55mm x 148mm", 55, 148),
    208: SecondarySizeInfo(208, "55mm x 173mm", 55, 173),
    209: SecondarySizeInfo(209, "52mm x 210mm", 52, 210),
    210: SecondarySizeInfo(210, "Kitchen", 210, 420),
    211: SecondarySizeInfo(211, "135mm x 205mm", 135, 205, 5.315, 8.071),
    212: SecondarySizeInfo(212, "125mm x 190mm", 125, 190, 4.921, 7.480),
    213: SecondarySizeInfo(213, "115mm x 185mm", 115, 185, 4.528, 7.283),
    214: SecondarySizeInfo(214, "114mm x 172mm", 114, 172, 4.488, 6.772),
    215: SecondarySizeInfo(215, "148mm x 156mm", 148, 156, 5.827, 6.142),
    216: SecondarySizeInfo(216, "294mm x 300mm", 294, 300, 11.575, 11.811),
    217: SecondarySizeInfo(217, "120mm x 190mm", 120, 190, 4.724, 7.480),
    218: SecondarySizeInfo(218, "5.5\" x 10\"", 139.7, 254, 5.5, 10),
    219: SecondarySizeInfo(219, "6\" x 8\"", 152.4, 203.2, 6, 8),
    220: SecondarySizeInfo(220, "12\" x 16\"", 304.8, 406.4, 12, 16),
    221: SecondarySizeInfo(221, "135mm x 210mm", 135, 210, 5.32, 8.27),
    222: SecondarySizeInfo(222, "160mm x 240mm", 160, 240, 6.30, 9.45),
    223: SecondarySizeInfo(223, "Royal+ (168mm x 240mm)", 168, 240, 6.61, 9.45),
    224: SecondarySizeInfo(224, "193mm x 260mm", 193, 260, 7.60, 10.24),
    225: SecondarySizeInfo(225, "AB (210mm x 257mm)", 210, 257, 8.26, 10.11),
    226: SecondarySizeInfo(226, "B5 (182mm x 257mm)", 182, 257, 7.16, 10.11),
    227: SecondarySizeInfo(227, "JIS B5 (179mm x 252mm)", 179, 252, 7.04, 9.92),
    228: SecondarySizeInfo(228, "JIS B40 (103mm x 182mm)", 103, 182, 4.05, 7.16),
    229: SecondarySizeInfo(229, "Han 46 (128mm x 188mm)", 128, 188, 5.03, 7.40),
    230: SecondarySizeInfo(230, "Hobonichi (110mm x 148mm)", 110, 148, 4.33, 5.82),
    231: SecondarySizeInfo(231, "95mm x 210mm", 95, 210, 3.74, 8.27),
}


def get_din_sizes() -> Tuple[Tuple[int, int], ...]:
    return DIN_SIZES


def get_b_sizes() -> Tuple[Tuple[int, int], ...]:
    return B_SIZES


def _is_landscape_from_orientation(width: float, height: float, orientation: Orientation | None) -> bool:
    return (orientation is None and width > height) or (orientation is not None and orientation == Orientation.LANDSCAPE)


def get_din_size_index(
    w: float,
    h: float,
    orientation: Orientation | None = None,
) -> int:
    is_landscape = _is_landscape_from_orientation(w, h, orientation)
    width_index = 1 if is_landscape else 0
    height_index = 0 if is_landscape else 1

    for i in range(len(DIN_SIZES) - 1, -1, -1):
        p_w, p_h = DIN_SIZES[i][width_index], DIN_SIZES[i][height_index]
        w_ratio = abs((p_w - w) / p_w * 100)
        h_ratio = (p_h - h) / p_h * 100
        if w_ratio <= PAGE_SIZE_TOLERANCE:
            if abs(h_ratio) <= PAGE_SIZE_TOLERANCE:
                return i
            if 0 <= h_ratio <= PAGE_HEIGHT_TOLERANCE:
                return i

    # spine detection
    if (h / w) > 8 or (w / h) > 8:
        return PAGE_SIZE_SPINE

    return PAGE_SIZE_UNKNOWN


def get_din_size_from_layout(
    sides: int,
    simple_fold: SimpleFold,
    w: float,
    h: float,
    orientation: Orientation | None = None,
) -> int:
    fold_factor_w = 4 if simple_fold == SimpleFold.CROSS else 2
    fold_factor_h = 2 if simple_fold == SimpleFold.CROSS else 1
    adj_w = w / sides * fold_factor_w
    adj_h = h / fold_factor_h
    return get_din_size_index(adj_w, adj_h, orientation)


def get_din_dimensions(size_idx: int, orientation: Orientation = Orientation.PORTRAIT) -> Tuple[float, float]:
    o = orientation
    return float(DIN_SIZES[size_idx][o.value]), float(DIN_SIZES[size_idx][o.value ^ 1])

def validate_secondary_size(
    secondary_id: int,
    w: float,
    h: float,
    orientation: Orientation | None = None,
) -> bool:
    is_landscape = _is_landscape_from_orientation(w, h, orientation)
    width_index = 1 if is_landscape else 0
    height_index = 0 if is_landscape else 1

    info = _secondary_info_map[secondary_id]
    p = (info.width, info.height)
    type_width = p[width_index]
    type_height = p[height_index]
    return abs(type_width - w) < SECONDARY_SIZE_TOLERANCE and abs(type_height - h) < SECONDARY_SIZE_TOLERANCE


def validate_secondary_size_from_layout(
    secondary_id: int,
    sides: int,
    simple_fold: SimpleFold,
    w: float,
    h: float,
    orientation: Orientation | None = None,
) -> bool:
    fold_factor_w = 4 if simple_fold == SimpleFold.CROSS else 2
    fold_factor_h = 2 if simple_fold == SimpleFold.CROSS else 1
    adj_w = w / sides * fold_factor_w
    adj_h = h / fold_factor_h
    return validate_secondary_size(secondary_id, adj_w, adj_h, orientation)


def get_secondary_name(secondary_id: int) -> str:
    return _secondary_info_map[secondary_id].name


def get_secondary_size(
    secondary_id: int,
    orientation: Orientation | None = None,
    size_format: UnitFormat = UnitFormat.METRIC,
) -> Tuple[float, float]:
    if orientation is None:
        orientation = Orientation.PORTRAIT
    info = _secondary_info_map[secondary_id]
    w, h = info.get_dims(size_format)
    p = (w, h)
    return float(p[orientation.value]), float(p[orientation.value ^ 1])


def is_dim_of_size(size_idx: int, w: float, h: float, orientation: Orientation) -> bool:
    dw, dh = get_din_dimensions(size_idx, orientation)
    return abs(w - dw) < INACCURACY and abs(h - dh) < INACCURACY

def mm_to_inch(size: float) -> float:
    return size / MM_IN_INCH


def mm_to_point(size: float) -> float:
    return size * MM_IN_POINT


def mm_to_300dpi_pixel(size: float) -> int:
    return int(size * MM_IN_300_DPI_PIXEL)


def inch_to_mm(size: float) -> float:
    return size * MM_IN_INCH


def inch_to_point(size: float) -> float:
    return size * INCH_IN_POINT


def inch_to_300dpi_pixel(size: float) -> int:
    return int(size * INCH_IN_300_DPI_PIXEL)

def calculate_max_vertical_items_per_plate(machine_width: float, job_width: float) -> int:
    return int(machine_width / (job_width + (DEFAULT_BLEED * 2)))


def calculate_max_horizontal_items_per_plate(machine_height: float, job_height: float) -> int:
    return int(machine_height / (job_height + (DEFAULT_BLEED * 2)))

def get_b_format_from_custom_size(
    width: float,
    height: float,
    size_format: UnitFormat,
    is_offset_optimized: bool,
    is_perfect_bound: bool,
) -> int:
    # Convert to MM & Switch Orientation To Portrait
    w = min(width, height) * (MM_IN_INCH if size_format == UnitFormat.IMPERIAL else 1.0)
    h = max(width, height) * (MM_IN_INCH if size_format == UnitFormat.IMPERIAL else 1.0)

    printable_w_desc = [125, 150, 217, 353, 500, 707, 1000]
    printable_h_desc = [176, 214, 300, 500, 707, 1000, 1414]

    if is_offset_optimized:
        printable_w_desc = [125, 170, 245, 348, 495, 702, 995]
        if is_perfect_bound:
            printable_h_desc = [172, 240, 304, 495, 702, 995, 1409]
        else:
            printable_h_desc = [172, 240, 348, 495, 702, 995, 1409]

    for i in range(7):
        if printable_w_desc[i] > w and printable_h_desc[i] > h:
            return 6 - i

    return 0



from __future__ import annotations

from enum import Enum

from mixam_sdk.item_specification.models.value_based import ValueBased


class SubstrateColour(ValueBased, Enum):

    NONE = 0
    BLACK = 1
    RED = 2
    ORANGE = 3
    YELLOW = 4
    GREEN = 5
    BLUE = 6
    INDIGO = 7
    VIOLET = 8
    PURPLE = 9
    PINK = 10
    TURQUOISE = 11
    SILVER = 12
    GOLD = 13
    GREY = 14
    ROSE = 15
    CREAM = 16
    DARK_GREEN = 100
    RUBY = 101
    BUTTERMILK = 102
    SAPPHIRE_BLUE = 103
    GRAPHITE = 104
    PASTEL_PINK = 105
    PASTEL_GREEN = 106
    LEAFBIRD_GREEN = 107
    PUFFIN_BLUE = 108
    SKYLARK_VIOLET = 109

    def __init__(self, value: int):
        self._value_ = value

    def get_value(self) -> int:
        return self.value



from __future__ import annotations

from enum import Enum

from mixam_sdk.item_specification.models.substrate_weight import SubstrateWeight, SubstrateWeightType


class SubstrateWeights:
    class Uk:
        class Silk(Enum):
            GSM_90 = SubstrateWeight(0, 90, 78.0)
            GSM_110 = SubstrateWeight(18, 110, 92.0)
            GSM_115 = SubstrateWeight(2, 115, 97.0)
            GSM_130 = SubstrateWeight(3, 130, 109.0)
            GSM_150 = SubstrateWeight(4, 150, 126.0)
            GSM_170 = SubstrateWeight(5, 170, 145.0)
            GSM_200 = SubstrateWeight(14, 200, 172.0)
            GSM_250 = SubstrateWeight(7, 250, 225.0)
            GSM_240 = SubstrateWeight(16, 240, 200.0)
            GSM_300 = SubstrateWeight(8, 300, 282.0)
            GSM_350 = SubstrateWeight(9, 350, 347.0)
            GSM_400 = SubstrateWeight(13, 400, 412.0)
            GSM_450 = SubstrateWeight(15, 450, 477.0)
            GSM_100 = SubstrateWeight(1, 100, 85.0)
            GSM_70 = SubstrateWeight(17, 70, 61.0)
            GSM_80 = SubstrateWeight(19, 80, 69.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            GSM_74 = SubstrateWeight(11, 74, 50.0)
            GSM_104 = SubstrateWeight(12, 104, 70.0)
            GSM_90 = SubstrateWeight(0, 90, 70.0)
            GSM_115 = SubstrateWeight(2, 115, 85.0)
            GSM_130 = SubstrateWeight(3, 130, 94.0)
            GSM_150 = SubstrateWeight(4, 150, 110.0)
            GSM_170 = SubstrateWeight(5, 170, 128.0)
            GSM_200 = SubstrateWeight(14, 200, 152.0)
            GSM_250 = SubstrateWeight(7, 250, 190.0)
            GSM_300 = SubstrateWeight(8, 300, 234.0)
            GSM_350 = SubstrateWeight(9, 350, 280.0)
            GSM_400 = SubstrateWeight(13, 400, 326.0)
            GSM_70 = SubstrateWeight(17, 70, 54.0)
            GSM_80 = SubstrateWeight(19, 80, 62.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            GSM_80 = SubstrateWeight(116, 80, 54.0)
            GSM_90 = SubstrateWeight(100, 90, 97.0)
            GSM_100 = SubstrateWeight(111, 100, 115.0)
            GSM_110 = SubstrateWeight(119, 110, 127.0)
            GSM_120 = SubstrateWeight(112, 120, 138.0)
            GSM_150 = SubstrateWeight(104, 150, 173.0)
            GSM_170 = SubstrateWeight(105, 170, 196.0)
            GSM_200 = SubstrateWeight(113, 200, 230.0)
            GSM_250 = SubstrateWeight(107, 250, 287.0)
            GSM_300 = SubstrateWeight(108, 300, 345.0)
            GSM_350 = SubstrateWeight(109, 350, 403.0)
            GSM_60 = SubstrateWeight(117, 60, 64.0)
            GSM_400 = SubstrateWeight(118, 400, 451.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PostcardBoard(Enum):
            GSM_280 = SubstrateWeight(11001, 280, 375.0)
            GSM_300 = SubstrateWeight(11003, 300, 438.0)
            GSM_350 = SubstrateWeight(11002, 350, 438.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            GSM_80 = SubstrateWeight(7001, 80, 100.0)
            GSM_100 = SubstrateWeight(7006, 100, 125.0)
            GSM_120 = SubstrateWeight(7007, 120, 150.0)
            GSM_135 = SubstrateWeight(7008, 135, 175.0)
            GSM_150 = SubstrateWeight(7009, 150, 188.0)
            GSM_170 = SubstrateWeight(7002, 170, 213.0)
            GSM_190 = SubstrateWeight(7010, 190, 237.0)
            GSM_250 = SubstrateWeight(7003, 250, 288.0)
            GSM_300 = SubstrateWeight(7005, 300, 345.0)
            GSM_350 = SubstrateWeight(7004, 350, 403.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledNatural(Enum):
            GSM_80 = SubstrateWeight(8001, 80, 103.0)
            GSM_100 = SubstrateWeight(8007, 100, 134.0)
            GSM_115 = SubstrateWeight(8006, 115, 154.0)
            GSM_135 = SubstrateWeight(8008, 135, 173.0)
            GSM_170 = SubstrateWeight(8002, 170, 212.0)
            GSM_250 = SubstrateWeight(8003, 250, 325.0)
            GSM_300 = SubstrateWeight(8005, 300, 390.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledSilk(Enum):
            GSM_100 = SubstrateWeight(45003, 100, 88.0)
            GSM_115 = SubstrateWeight(45004, 115, 90.0)
            GSM_130 = SubstrateWeight(45005, 130, 103.0)
            GSM_150 = SubstrateWeight(45006, 150, 118.0)
            GSM_170 = SubstrateWeight(45007, 170, 135.0)
            GSM_200 = SubstrateWeight(45008, 200, 165.0)
            GSM_250 = SubstrateWeight(45009, 250, 215.0)
            GSM_300 = SubstrateWeight(45010, 300, 275.0)
            GSM_350 = SubstrateWeight(45011, 350, 325.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Polyester(Enum):
            GSM_260 = SubstrateWeight(55000, 260, 220.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class MattePaper(Enum):
            GSM_74 = SubstrateWeight(62000, 74, 3.8)
            GSM_104 = SubstrateWeight(62001, 104, 3.8)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PremiumWhite(Enum):
            GSM_80 = SubstrateWeight(63000, 80, 3.9)
            GSM_90 = SubstrateWeight(63001, 90, 4.2)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Creme(Enum):
            GSM_70 = SubstrateWeight(60002, 70, 50.0)
            GSM_74 = SubstrateWeight(60000, 74, 50.0)
            GSM_80 = SubstrateWeight(60001, 80, 54.0)
            GSM_90 = SubstrateWeight(60003, 90, 58.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class EPhotoSilkLustre(Enum):
            GSM_190 = SubstrateWeight(78001, 190, 170.0)
            GSM_260 = SubstrateWeight(78002, 260, 225.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalTexturedMatt(Enum):
            GSM_240 = SubstrateWeight(81001, 240, 200.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalUncoated(Enum):
            GSM_300 = SubstrateWeight(80001, 300, 282.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalMatt(Enum):
            GSM_240 = SubstrateWeight(79001, 240, 200.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class WrappedGreyboard(Enum):
            GSM_650 = SubstrateWeight(83000, 650, 880000.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Linen(Enum):
            GSM_115 = SubstrateWeight(12000, 115, 160.0)
            GSM_125 = SubstrateWeight(12001, 125, 180.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PearlPolar(Enum):
            GSM_125 = SubstrateWeight(16000, 125, 150.0)
            GSM_300 = SubstrateWeight(16001, 300, 360.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PearlOyster(Enum):
            GSM_125 = SubstrateWeight(17000, 125, 150.0)
            GSM_300 = SubstrateWeight(17001, 300, 360.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class IceGold(Enum):
            GSM_300 = SubstrateWeight(41000, 300, 400.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class TintorettoGesso(Enum):
            GSM_140 = SubstrateWeight(36000, 140, 196.0)
            GSM_150 = SubstrateWeight(36001, 150, 210.0)
            GSM_250 = SubstrateWeight(36002, 250, 358.0)
            GSM_300 = SubstrateWeight(36003, 300, 429.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RivesShetland(Enum):
            GSM_250 = SubstrateWeight(43000, 250, 357.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Aquerello(Enum):
            GSM_160 = SubstrateWeight(86000, 160, 232.0)
            GSM_280 = SubstrateWeight(86001, 280, 406.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class FrescoGesso(Enum):
            GSM_300 = SubstrateWeight(42000, 300, 435.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Nettuno(Enum):
            GSM_140 = SubstrateWeight(87000, 140, 210.0)
            GSM_280 = SubstrateWeight(87001, 280, 420.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class LuxLayeredKraft(Enum):
            GSM_810 = SubstrateWeight(88000, 810, 1000.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class LuxLayeredWhite(Enum):
            GSM_810 = SubstrateWeight(89000, 810, 1000.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Kraft(Enum):
            GSM_120 = SubstrateWeight(39000, 120, 170.0)
            GSM_300 = SubstrateWeight(39001, 300, 380.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Laid(Enum):
            GSM_120 = SubstrateWeight(85000, 120, 171.0)
            GSM_300 = SubstrateWeight(85001, 300, 430.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Etching(Enum):
            GSM_300 = SubstrateWeight(90000, 300, 500.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RagPearl(Enum):
            GSM_320 = SubstrateWeight(91000, 320, 480.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class Us:
        class Silk(Enum):
            LBS_80_COVER = SubstrateWeight(1, 216, 8.00, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(2, 271, 10.00, 100, SubstrateWeightType.COVER)
            LBS_110_COVER = SubstrateWeight(3, 297, 12.00, 110, SubstrateWeightType.COVER)
            LBS_130_COVER = SubstrateWeight(5, 350, 16.00, 130, SubstrateWeightType.COVER)
            LBS_70_TEXT = SubstrateWeight(7, 104, 3.50, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(8, 118, 4.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(9, 148, 5.00, 100, SubstrateWeightType.TEXT)
            LBS_60_TEXT = SubstrateWeight(17, 104, 3.10, 60, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            LBS_50_COVER = SubstrateWeight(11, 74, 50.00, 50, SubstrateWeightType.COVER)
            LBS_70_COVER = SubstrateWeight(12, 104, 70.00, 70, SubstrateWeightType.COVER)
            LBS_80_COVER = SubstrateWeight(1, 216, 8.00, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(2, 271, 10.00, 100, SubstrateWeightType.COVER)
            LBS_110_COVER = SubstrateWeight(3, 297, 12.00, 110, SubstrateWeightType.COVER)
            LBS_130_COVER = SubstrateWeight(5, 350, 16.00, 130, SubstrateWeightType.COVER)
            LBS_70_TEXT = SubstrateWeight(7, 104, 3.50, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(8, 118, 4.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(9, 148, 5.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            LBS_65_COVER = SubstrateWeight(101, 176, 9.50, 65, SubstrateWeightType.COVER)
            LBS_80_COVER = SubstrateWeight(102, 216, 11.50, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(103, 270, 14.00, 100, SubstrateWeightType.COVER)
            LBS_120_COVER = SubstrateWeight(104, 325, 17.00, 120, SubstrateWeightType.COVER)
            LBS_40_TEXT = SubstrateWeight(117, 60, 3.4, 40, SubstrateWeightType.TEXT)
            LBS_50_TEXT = SubstrateWeight(105, 74, 4.00, 50, SubstrateWeightType.TEXT)
            LBS_60_TEXT = SubstrateWeight(106, 89, 4.50, 60, SubstrateWeightType.TEXT)
            LBS_70_TEXT = SubstrateWeight(107, 104, 5.00, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(108, 118, 6.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(109, 148, 7.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class MattePaper(Enum):
            LBS_50_TEXT = SubstrateWeight(62000, 74, 50.00, 50, SubstrateWeightType.TEXT)
            LBS_70_TEXT = SubstrateWeight(62001, 104, 70.00, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(62002, 120, 80.00, 80, SubstrateWeightType.TEXT)
            LBS_80_COVER = SubstrateWeight(62003, 216, 9.20, 80, SubstrateWeightType.COVER)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class None_(Enum):
            LBS_0_TEXT = SubstrateWeight(0, 0, 0.00, 0, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PolyfillBag(Enum):
            LBS_1_TEXT = SubstrateWeight(76001, 1, 0.10, 1, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PremiumWhite(Enum):
            LBS_54_TEXT = SubstrateWeight(63000, 80, 3.80, 54, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            LBS_60_TEXT = SubstrateWeight(7001, 89, 4.50, 60, SubstrateWeightType.TEXT)
            LBS_70_TEXT = SubstrateWeight(7002, 104, 5.00, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(7003, 118, 6.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(7004, 148, 7.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class UncoatedCremePaper(Enum):
            LBS_60_TEXT = SubstrateWeight(35000, 89, 4.00, 60, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class EPhotoPaper(Enum):
            LBS_110_COVER = SubstrateWeight(77000, 222, 8.35, 110, SubstrateWeightType.COVER)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class Ca:
        class Silk(Enum):
            LBS_80_COVER = SubstrateWeight(1, 216, 8.00, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(2, 271, 10.00, 100, SubstrateWeightType.COVER)
            LBS_110_COVER = SubstrateWeight(3, 297, 12.00, 110, SubstrateWeightType.COVER)
            LBS_130_COVER = SubstrateWeight(5, 350, 16.00, 130, SubstrateWeightType.COVER)
            LBS_70_TEXT = SubstrateWeight(7, 104, 3.50, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(8, 118, 4.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(9, 148, 5.00, 100, SubstrateWeightType.TEXT)
            LBS_60_TEXT = SubstrateWeight(17, 104, 3.10, 60, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            LBS_50_COVER = SubstrateWeight(11, 74, 50.00, 50, SubstrateWeightType.COVER)
            LBS_70_COVER = SubstrateWeight(12, 104, 70.00, 70, SubstrateWeightType.COVER)
            LBS_80_COVER = SubstrateWeight(1, 216, 8.00, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(2, 271, 10.00, 100, SubstrateWeightType.COVER)
            LBS_110_COVER = SubstrateWeight(3, 297, 12.00, 110, SubstrateWeightType.COVER)
            LBS_130_COVER = SubstrateWeight(5, 350, 16.00, 130, SubstrateWeightType.COVER)
            LBS_70_TEXT = SubstrateWeight(7, 104, 3.50, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(8, 118, 4.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(9, 148, 5.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            LBS_65_COVER = SubstrateWeight(101, 176, 9.50, 65, SubstrateWeightType.COVER)
            LBS_80_COVER = SubstrateWeight(102, 216, 11.50, 80, SubstrateWeightType.COVER)
            LBS_100_COVER = SubstrateWeight(103, 270, 14.00, 100, SubstrateWeightType.COVER)
            LBS_120_COVER = SubstrateWeight(104, 325, 17.00, 120, SubstrateWeightType.COVER)
            LBS_50_TEXT = SubstrateWeight(105, 74, 4.00, 50, SubstrateWeightType.TEXT)
            LBS_60_TEXT = SubstrateWeight(106, 89, 4.50, 60, SubstrateWeightType.TEXT)
            LBS_70_TEXT = SubstrateWeight(107, 104, 5.00, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(108, 118, 6.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(109, 148, 7.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PhotoLustre(Enum):
            LBS_110_COVER = SubstrateWeight(82000, 222, 8.35, 110, SubstrateWeightType.COVER)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            LBS_60_TEXT = SubstrateWeight(7001, 89, 4.50, 60, SubstrateWeightType.TEXT)
            LBS_70_TEXT = SubstrateWeight(7002, 104, 5.00, 70, SubstrateWeightType.TEXT)
            LBS_80_TEXT = SubstrateWeight(7003, 118, 6.00, 80, SubstrateWeightType.TEXT)
            LBS_100_TEXT = SubstrateWeight(7004, 148, 7.00, 100, SubstrateWeightType.TEXT)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class Au:
        class Silk(Enum):
            GSM_90 = SubstrateWeight(0, 90, 78.0)
            GSM_115 = SubstrateWeight(2, 115, 97.0)
            GSM_130 = SubstrateWeight(3, 130, 109.0)
            GSM_150 = SubstrateWeight(4, 150, 126.0)
            GSM_170 = SubstrateWeight(5, 170, 145.0)
            GSM_200 = SubstrateWeight(14, 200, 172.0)
            GSM_250 = SubstrateWeight(7, 250, 225.0)
            GSM_300 = SubstrateWeight(8, 300, 282.0)
            GSM_350 = SubstrateWeight(9, 350, 347.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            GSM_90 = SubstrateWeight(0, 90, 70.0)
            GSM_115 = SubstrateWeight(2, 115, 85.0)
            GSM_130 = SubstrateWeight(3, 130, 94.0)
            GSM_150 = SubstrateWeight(4, 150, 110.0)
            GSM_170 = SubstrateWeight(5, 170, 128.0)
            GSM_200 = SubstrateWeight(14, 200, 152.0)
            GSM_250 = SubstrateWeight(7, 250, 190.0)
            GSM_300 = SubstrateWeight(8, 300, 234.0)
            GSM_350 = SubstrateWeight(9, 350, 280.0)
            GSM_74 = SubstrateWeight(11, 74, 50.0)
            GSM_104 = SubstrateWeight(12, 104, 70.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            GSM_90 = SubstrateWeight(100, 90, 97.0)
            GSM_100 = SubstrateWeight(111, 100, 115.0)
            GSM_120 = SubstrateWeight(112, 120, 138.0)
            GSM_150 = SubstrateWeight(104, 150, 173.0)
            GSM_170 = SubstrateWeight(105, 170, 196.0)
            GSM_200 = SubstrateWeight(113, 200, 230.0)
            GSM_250 = SubstrateWeight(107, 250, 287.0)
            GSM_300 = SubstrateWeight(108, 300, 345.0)
            GSM_350 = SubstrateWeight(109, 350, 403.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Creme(Enum):
            GSM_70 = SubstrateWeight(60002, 70, 50.0)
            GSM_74 = SubstrateWeight(60000, 74, 50.0)
            GSM_80 = SubstrateWeight(60001, 80, 54.0)
            GSM_90 = SubstrateWeight(60003, 90, 58.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class MattePaper(Enum):
            GSM_74 = SubstrateWeight(62000, 74, 3.8)
            GSM_104 = SubstrateWeight(62001, 104, 3.8)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledSilk(Enum):
            GSM_100 = SubstrateWeight(45003, 100, 88.0)
            GSM_115 = SubstrateWeight(45004, 115, 90.0)
            GSM_130 = SubstrateWeight(45005, 130, 103.0)
            GSM_150 = SubstrateWeight(45006, 150, 118.0)
            GSM_170 = SubstrateWeight(45007, 170, 135.0)
            GSM_200 = SubstrateWeight(45008, 200, 165.0)
            GSM_250 = SubstrateWeight(45009, 250, 215.0)
            GSM_300 = SubstrateWeight(45010, 300, 275.0)
            GSM_350 = SubstrateWeight(45011, 350, 325.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            GSM_90 = SubstrateWeight(7001, 90, 113.0)
            GSM_100 = SubstrateWeight(7006, 100, 125.0)
            GSM_120 = SubstrateWeight(7007, 120, 150.0)
            GSM_135 = SubstrateWeight(7008, 135, 175.0)
            GSM_150 = SubstrateWeight(7009, 150, 188.0)
            GSM_160 = SubstrateWeight(7011, 160, 201.0)
            GSM_170 = SubstrateWeight(7002, 170, 213.0)
            GSM_190 = SubstrateWeight(7010, 190, 237.0)
            GSM_250 = SubstrateWeight(7003, 250, 288.0)
            GSM_300 = SubstrateWeight(7005, 300, 345.0)
            GSM_350 = SubstrateWeight(7004, 350, 403.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class De:
        class Silk(Enum):
            GSM_90 = SubstrateWeight(0, 90, 78.0)
            GSM_110 = SubstrateWeight(18, 110, 92.0)
            GSM_115 = SubstrateWeight(2, 115, 97.0)
            GSM_135 = SubstrateWeight(3, 135, 109.0)
            GSM_150 = SubstrateWeight(4, 150, 126.0)
            GSM_170 = SubstrateWeight(5, 170, 145.0)
            GSM_200 = SubstrateWeight(14, 200, 172.0)
            GSM_250 = SubstrateWeight(7, 250, 225.0)
            GSM_240 = SubstrateWeight(16, 240, 200.0)
            GSM_300 = SubstrateWeight(8, 300, 282.0)
            GSM_350 = SubstrateWeight(9, 350, 347.0)
            GSM_400 = SubstrateWeight(13, 400, 412.0)
            GSM_450 = SubstrateWeight(15, 450, 477.0)
            GSM_100 = SubstrateWeight(1, 100, 85.0)
            GSM_70 = SubstrateWeight(19, 70, 60.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            GSM_74 = SubstrateWeight(11, 74, 50.0)
            GSM_104 = SubstrateWeight(12, 104, 70.0)
            GSM_90 = SubstrateWeight(0, 90, 70.0)
            GSM_115 = SubstrateWeight(2, 115, 85.0)
            GSM_135 = SubstrateWeight(3, 135, 94.0)
            GSM_150 = SubstrateWeight(4, 150, 110.0)
            GSM_170 = SubstrateWeight(5, 170, 128.0)
            GSM_200 = SubstrateWeight(14, 200, 152.0)
            GSM_250 = SubstrateWeight(7, 250, 190.0)
            GSM_300 = SubstrateWeight(8, 300, 234.0)
            GSM_350 = SubstrateWeight(9, 350, 280.0)
            GSM_400 = SubstrateWeight(13, 400, 326.0)
            GSM_100 = SubstrateWeight(1, 100, 77.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            GSM_80 = SubstrateWeight(116, 80, 100.0)
            GSM_90 = SubstrateWeight(100, 90, 110.0)
            GSM_100 = SubstrateWeight(111, 100, 115.0)
            GSM_110 = SubstrateWeight(119, 110, 127.0)
            GSM_120 = SubstrateWeight(112, 120, 138.0)
            GSM_150 = SubstrateWeight(104, 150, 173.0)
            GSM_170 = SubstrateWeight(105, 170, 196.0)
            GSM_200 = SubstrateWeight(113, 200, 230.0)
            GSM_250 = SubstrateWeight(107, 250, 287.0)
            GSM_300 = SubstrateWeight(108, 300, 345.0)
            GSM_350 = SubstrateWeight(109, 350, 403.0)
            GSM_400 = SubstrateWeight(110, 400, 453.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledNatural(Enum):
            GSM_80 = SubstrateWeight(8001, 80, 103.0)
            GSM_90 = SubstrateWeight(8004, 90, 118.0)
            GSM_100 = SubstrateWeight(8007, 100, 134.0)
            GSM_120 = SubstrateWeight(8006, 120, 154.0)
            GSM_135 = SubstrateWeight(8008, 135, 173.0)
            GSM_160 = SubstrateWeight(8002, 160, 212.0)
            GSM_250 = SubstrateWeight(8003, 250, 325.0)
            GSM_300 = SubstrateWeight(8005, 300, 390.0)
            GSM_400 = SubstrateWeight(8009, 400, 450.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            GSM_80 = SubstrateWeight(7001, 80, 100.0)
            GSM_100 = SubstrateWeight(7006, 100, 125.0)
            GSM_120 = SubstrateWeight(7007, 120, 150.0)
            GSM_135 = SubstrateWeight(7008, 135, 175.0)
            GSM_150 = SubstrateWeight(7009, 150, 188.0)
            GSM_170 = SubstrateWeight(7002, 170, 213.0)
            GSM_400 = SubstrateWeight(7010, 400, 450.0)
            GSM_250 = SubstrateWeight(7003, 250, 288.0)
            GSM_300 = SubstrateWeight(7005, 300, 345.0)
            GSM_350 = SubstrateWeight(7004, 350, 403.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Polyester(Enum):
            GSM_190 = SubstrateWeight(56000, 190, 150.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Creme(Enum):
            GSM_70 = SubstrateWeight(60002, 70, 50.0)
            GSM_74 = SubstrateWeight(60000, 74, 50.0)
            GSM_80 = SubstrateWeight(60001, 80, 54.0)
            GSM_90 = SubstrateWeight(60003, 90, 58.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PostcardBoard(Enum):
            GSM_280 = SubstrateWeight(11001, 280, 375.0)
            GSM_300 = SubstrateWeight(11003, 300, 438.0)
            GSM_350 = SubstrateWeight(11002, 350, 438.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Affiche(Enum):
            GSM_115 = SubstrateWeight(10001, 115, 97.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class Ie:
        class Silk(Enum):
            GSM_90 = SubstrateWeight(0, 90, 78.0)
            GSM_110 = SubstrateWeight(18, 110, 92.0)
            GSM_115 = SubstrateWeight(2, 115, 97.0)
            GSM_135 = SubstrateWeight(3, 135, 109.0)
            GSM_150 = SubstrateWeight(4, 150, 126.0)
            GSM_170 = SubstrateWeight(5, 170, 145.0)
            GSM_200 = SubstrateWeight(14, 200, 172.0)
            GSM_250 = SubstrateWeight(7, 250, 225.0)
            GSM_240 = SubstrateWeight(16, 240, 200.0)
            GSM_300 = SubstrateWeight(8, 300, 282.0)
            GSM_350 = SubstrateWeight(9, 350, 347.0)
            GSM_400 = SubstrateWeight(13, 400, 412.0)
            GSM_450 = SubstrateWeight(15, 450, 477.0)
            GSM_100 = SubstrateWeight(1, 100, 85.0)
            GSM_70 = SubstrateWeight(19, 70, 60.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            GSM_74 = SubstrateWeight(11, 74, 50.0)
            GSM_104 = SubstrateWeight(12, 104, 70.0)
            GSM_90 = SubstrateWeight(0, 90, 70.0)
            GSM_115 = SubstrateWeight(2, 115, 85.0)
            GSM_135 = SubstrateWeight(3, 135, 94.0)
            GSM_150 = SubstrateWeight(4, 150, 110.0)
            GSM_170 = SubstrateWeight(5, 170, 128.0)
            GSM_200 = SubstrateWeight(14, 200, 152.0)
            GSM_250 = SubstrateWeight(7, 250, 190.0)
            GSM_300 = SubstrateWeight(8, 300, 234.0)
            GSM_350 = SubstrateWeight(9, 350, 280.0)
            GSM_400 = SubstrateWeight(13, 400, 326.0)
            GSM_100 = SubstrateWeight(1, 100, 77.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            GSM_80 = SubstrateWeight(116, 80, 54.0)
            GSM_90 = SubstrateWeight(100, 90, 97.0)
            GSM_100 = SubstrateWeight(111, 100, 115.0)
            GSM_110 = SubstrateWeight(119, 110, 127.0)
            GSM_120 = SubstrateWeight(112, 120, 138.0)
            GSM_150 = SubstrateWeight(104, 150, 173.0)
            GSM_170 = SubstrateWeight(105, 170, 196.0)
            GSM_200 = SubstrateWeight(113, 200, 230.0)
            GSM_250 = SubstrateWeight(107, 250, 287.0)
            GSM_300 = SubstrateWeight(108, 300, 345.0)
            GSM_350 = SubstrateWeight(109, 350, 403.0)
            GSM_400 = SubstrateWeight(110, 400, 453.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledNatural(Enum):
            GSM_80 = SubstrateWeight(8001, 80, 103.0)
            GSM_90 = SubstrateWeight(8004, 90, 118.0)
            GSM_100 = SubstrateWeight(8007, 100, 134.0)
            GSM_120 = SubstrateWeight(8006, 120, 154.0)
            GSM_135 = SubstrateWeight(8008, 135, 173.0)
            GSM_160 = SubstrateWeight(8002, 160, 212.0)
            GSM_250 = SubstrateWeight(8003, 250, 325.0)
            GSM_300 = SubstrateWeight(8005, 300, 390.0)
            GSM_400 = SubstrateWeight(8009, 400, 450.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            GSM_80 = SubstrateWeight(7001, 80, 100.0)
            GSM_100 = SubstrateWeight(7006, 100, 125.0)
            GSM_120 = SubstrateWeight(7007, 120, 150.0)
            GSM_135 = SubstrateWeight(7008, 135, 175.0)
            GSM_150 = SubstrateWeight(7009, 150, 188.0)
            GSM_170 = SubstrateWeight(7002, 170, 213.0)
            GSM_190 = SubstrateWeight(7010, 190, 237.0)
            GSM_250 = SubstrateWeight(7003, 250, 288.0)
            GSM_300 = SubstrateWeight(7005, 300, 345.0)
            GSM_350 = SubstrateWeight(7004, 350, 403.0)
            GSM_400 = SubstrateWeight(7010, 400, 450.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Polyester(Enum):
            GSM_190 = SubstrateWeight(56000, 190, 150.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Creme(Enum):
            GSM_70 = SubstrateWeight(60002, 70, 50.0)
            GSM_74 = SubstrateWeight(60000, 74, 50.0)
            GSM_80 = SubstrateWeight(60001, 80, 54.0)
            GSM_90 = SubstrateWeight(60003, 90, 58.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PostcardBoard(Enum):
            GSM_280 = SubstrateWeight(11001, 280, 375.0)
            GSM_300 = SubstrateWeight(11003, 300, 438.0)
            GSM_350 = SubstrateWeight(11002, 350, 438.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Affiche(Enum):
            GSM_115 = SubstrateWeight(10001, 115, 97.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

    class Jp:
        class Silk(Enum):
            GSM_110 = SubstrateWeight(18, 110, 92.0)
            GSM_115 = SubstrateWeight(2, 115, 97.0)
            GSM_130 = SubstrateWeight(3, 130, 109.0)
            GSM_150 = SubstrateWeight(4, 150, 126.0)
            GSM_170 = SubstrateWeight(5, 170, 145.0)
            GSM_200 = SubstrateWeight(14, 200, 172.0)
            GSM_250 = SubstrateWeight(7, 250, 225.0)
            GSM_240 = SubstrateWeight(16, 240, 200.0)
            GSM_300 = SubstrateWeight(8, 300, 282.0)
            GSM_350 = SubstrateWeight(9, 350, 347.0)
            GSM_400 = SubstrateWeight(13, 400, 412.0)
            GSM_450 = SubstrateWeight(15, 450, 477.0)
            GSM_100 = SubstrateWeight(1, 100, 85.0)
            GSM_128 = SubstrateWeight(19, 128, 109.0)
            GSM_105 = SubstrateWeight(20, 105, 92.0)
            GSM_157 = SubstrateWeight(21, 157, 126.0)
            GSM_186 = SubstrateWeight(22, 186, 172.0)
            GSM_209 = SubstrateWeight(23, 209, 180.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Gloss(Enum):
            GSM_74 = SubstrateWeight(11, 74, 50.0)
            GSM_104 = SubstrateWeight(12, 104, 70.0)
            GSM_115 = SubstrateWeight(2, 115, 85.0)
            GSM_130 = SubstrateWeight(3, 130, 94.0)
            GSM_150 = SubstrateWeight(4, 150, 110.0)
            GSM_170 = SubstrateWeight(5, 170, 128.0)
            GSM_200 = SubstrateWeight(14, 200, 152.0)
            GSM_250 = SubstrateWeight(7, 250, 190.0)
            GSM_300 = SubstrateWeight(8, 300, 234.0)
            GSM_350 = SubstrateWeight(9, 350, 280.0)
            GSM_400 = SubstrateWeight(13, 400, 326.0)
            GSM_105 = SubstrateWeight(15, 105, 85.0)
            GSM_128 = SubstrateWeight(16, 128, 94.0)
            GSM_157 = SubstrateWeight(17, 157, 110.0)
            GSM_186 = SubstrateWeight(18, 186, 128.0)
            GSM_209 = SubstrateWeight(19, 209, 152.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Uncoated(Enum):
            GSM_80 = SubstrateWeight(116, 80, 54.0)
            GSM_90 = SubstrateWeight(100, 90, 97.0)
            GSM_100 = SubstrateWeight(111, 100, 115.0)
            GSM_110 = SubstrateWeight(119, 110, 127.0)
            GSM_120 = SubstrateWeight(112, 120, 138.0)
            GSM_150 = SubstrateWeight(104, 150, 173.0)
            GSM_170 = SubstrateWeight(105, 170, 196.0)
            GSM_200 = SubstrateWeight(113, 200, 230.0)
            GSM_250 = SubstrateWeight(107, 250, 287.0)
            GSM_300 = SubstrateWeight(108, 300, 345.0)
            GSM_350 = SubstrateWeight(109, 350, 403.0)
            GSM_81 = SubstrateWeight(120, 81, 97.0)
            GSM_105 = SubstrateWeight(121, 105, 127.0)
            GSM_128 = SubstrateWeight(122, 128, 138.0)
            GSM_157 = SubstrateWeight(123, 157, 173.0)
            GSM_209 = SubstrateWeight(124, 209, 230.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PostcardBoard(Enum):
            GSM_280 = SubstrateWeight(11001, 280, 375.0)
            GSM_300 = SubstrateWeight(11003, 300, 438.0)
            GSM_350 = SubstrateWeight(11002, 350, 438.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledUncoated(Enum):
            GSM_80 = SubstrateWeight(7001, 80, 100.0)
            GSM_100 = SubstrateWeight(7006, 100, 125.0)
            GSM_120 = SubstrateWeight(7007, 120, 150.0)
            GSM_135 = SubstrateWeight(7008, 135, 175.0)
            GSM_150 = SubstrateWeight(7009, 150, 188.0)
            GSM_170 = SubstrateWeight(7002, 170, 213.0)
            GSM_190 = SubstrateWeight(7010, 190, 237.0)
            GSM_250 = SubstrateWeight(7003, 250, 288.0)
            GSM_300 = SubstrateWeight(7005, 300, 345.0)
            GSM_350 = SubstrateWeight(7004, 350, 403.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledNatural(Enum):
            GSM_80 = SubstrateWeight(8001, 80, 103.0)
            GSM_100 = SubstrateWeight(8007, 100, 134.0)
            GSM_115 = SubstrateWeight(8006, 115, 154.0)
            GSM_135 = SubstrateWeight(8008, 135, 173.0)
            GSM_170 = SubstrateWeight(8002, 170, 212.0)
            GSM_250 = SubstrateWeight(8003, 250, 325.0)
            GSM_300 = SubstrateWeight(8005, 300, 390.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class RecycledSilk(Enum):
            GSM_100 = SubstrateWeight(45003, 100, 88.0)
            GSM_115 = SubstrateWeight(45004, 115, 90.0)
            GSM_130 = SubstrateWeight(45005, 130, 103.0)
            GSM_150 = SubstrateWeight(45006, 150, 118.0)
            GSM_170 = SubstrateWeight(45007, 170, 135.0)
            GSM_200 = SubstrateWeight(45008, 200, 165.0)
            GSM_250 = SubstrateWeight(45009, 250, 215.0)
            GSM_300 = SubstrateWeight(45010, 300, 275.0)
            GSM_350 = SubstrateWeight(45011, 350, 325.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Polyester(Enum):
            GSM_260 = SubstrateWeight(55000, 260, 220.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class MattePaper(Enum):
            GSM_74 = SubstrateWeight(62000, 74, 3.8)
            GSM_104 = SubstrateWeight(62001, 104, 3.8)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class PremiumWhite(Enum):
            GSM_80 = SubstrateWeight(63000, 80, 3.9)
            GSM_90 = SubstrateWeight(63001, 90, 4.2)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Creme(Enum):
            GSM_70 = SubstrateWeight(60002, 70, 50.0)
            GSM_74 = SubstrateWeight(60000, 74, 50.0)
            GSM_80 = SubstrateWeight(60001, 80, 54.0)
            GSM_90 = SubstrateWeight(60003, 90, 58.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class EPhotoSilkLustre(Enum):
            GSM_190 = SubstrateWeight(78001, 190, 170.0)
            GSM_260 = SubstrateWeight(78002, 260, 225.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalTexturedMatt(Enum):
            GSM_240 = SubstrateWeight(81001, 240, 200.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalUncoated(Enum):
            GSM_300 = SubstrateWeight(80001, 300, 282.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class ArchivalMatt(Enum):
            GSM_240 = SubstrateWeight(79001, 240, 200.0)

            def get_values(self) -> SubstrateWeight:
                return self.value

        class Buckram(Enum):
            GSM_115 = SubstrateWeight(84000, 115, 185.0)

            def get_values(self) -> SubstrateWeight:
                return self.value



from __future__ import annotations

from typing import Annotated

from pydantic import BaseModel, Field, ConfigDict

from mixam_sdk.item_specification.enums.binding_type import BindingType
from mixam_sdk.item_specification.models.substrate import Substrate
from mixam_sdk.utils.enum_json import enum_by_name_or_value, enum_dump_name
from .binding_colour_option import BindingColourOption
from .colours_option import ColoursOption
from ..enums.trilean import Trilean


class BindingTypeOption(BaseModel):

    binding_type: Annotated[BindingType, enum_by_name_or_value(BindingType), enum_dump_name] = Field(
        alias="bindingType",
    )

    label: str = Field(
        alias="label",
    )

    value: int = Field(
        alias="value",
    )

    default_pages: int = Field(
        alias="defaultPages",
    )
    caliper: float = Field(
        alias="caliper",
    )

    separate_cover: Annotated[Trilean, enum_by_name_or_value(Trilean), enum_dump_name] = Field(
        alias="separateCover",
    )

    separate_cover_outer_colours_options: list[ColoursOption] = Field(
        default_factory=list,
        alias="separateCoverOuterColoursOptions",
    )

    separate_cover_inner_colours_options: list[ColoursOption] = Field(
        alias="separateCoverInnerColoursOptions",
    )

    pre_drilled_holes: Annotated[Trilean, enum_by_name_or_value(Trilean), enum_dump_name] = Field(
        alias="preDrilledHoles",
    )

    colour_options: list[BindingColourOption] = Field(
        default_factory=list,
        alias="colourOptions",
    )

    sewing: Annotated[Trilean, enum_by_name_or_value(Trilean), enum_dump_name] = Field(
        alias="sewing",
    )

    required_substrate: Substrate | None = Field(
        default=None,
        alias="requiredSubstrate",
    )

    santa_default: bool = Field(
        default=False,
        alias="santaDefault",
    )

    supports_end_papers: bool = Field(
        default=False,
        alias="supportsEndPapers",
    )

    supports_dust_jacket: bool = Field(
        default=False,
        alias="supportsDustJacket",
    )

    supports_ribbons: bool = Field(
        default=False,
        alias="supportsRibbons",
    )

    supports_head_and_tail_bands: bool = Field(
        default=False,
        alias="supportsHeadAndTailBands",
    )

    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
        frozen=True,
        strict=True,
        validate_assignment=True,
    )


__all__ = ["BindingTypeOption"]



from __future__ import annotations

from enum import Enum

from mixam_sdk.item_specification.models.value_based import ValueBased


class FrameDepth(ValueBased, Enum):

    UNSPECIFIED = 0
    MM_18 = 1
    MM_38 = 2

    def get_value(self) -> int:
        return int(self.value)



from __future__ import annotations

from enum import Enum

from mixam_sdk.item_specification.models.value_based import ValueBased


class Colours(ValueBased, Enum):

    NONE = 0
    GRAYSCALE = 1
    PROCESS = 5

    def get_value(self) -> int:
        return int(self.value)



from __future__ import annotations

from enum import Enum


class WeightUnit(Enum):
    GSM = "GSM"
    LBS_TEXT = "LBS_TEXT"
    LBS_COVER = "LBS_COVER"
    MICRONS = "MICRONS"
    SHIROKU_BAN = "SHIROKU_BAN"



from __future__ import annotations

from enum import Enum

from mixam_sdk.item_specification.models.value_based import ValueBased


class EndPaperColour(ValueBased, Enum):

    WHITE = 0
    BLACK = 1
    COLORPLAN_ADRIATIC = 10
    COLORPLAN_AMETHYST = 11
    COLORPLAN_AZURE_BLUE = 12
    COLORPLAN_BARK = 13
    COLORPLAN_BITTER_CHOCOLATE = 14
    COLORPLAN_BRIGHT_RED = 15
    COLORPLAN_BRIGHT_WHITE = 16
    COLORPLAN_CANDY_PINK = 17
    COLORPLAN_LIMESTONE_WHITE = 18
    COLORPLAN_CITRINE = 19
    COLORPLAN_CLARET = 20
    COLORPLAN_COBALT = 21
    COLORPLAN_COOL_BLUE = 22
    COLORPLAN_COOL_GREY = 23
    COLORPLAN_DARK_GREY = 24
    COLORPLAN_EBONY = 25
    COLORPLAN_EMERALD = 26
    COLORPLAN_FACTORY_YELLOW = 27
    COLORPLAN_FOREST = 28
    COLORPLAN_FUSCHIA_PINK = 29
    COLORPLAN_HARVEST = 30
    COLORPLAN_HOT_PINK = 31
    COLORPLAN_ICE_WHITE = 32
    COLORPLAN_LOCH_BLUE = 33
    COLORPLAN_LAVENDAR = 34
    COLORPLAN_LOCKWOOD_GREEN = 35
    COLORPLAN_MANDARIN = 36
    COLORPLAN_MARRS_GREEN = 37
    COLORPLAN_MID_GREEN = 38
    COLORPLAN_MIST = 39
    COLORPLAN_NATURAL = 40
    COLORPLAN_NEW_BLUE = 41
    COLORPLAN_NUBUCK_BROWN = 42
    COLORPLAN_PALE_GREY = 43
    COLORPLAN_PARK_GREEN = 44
    COLORPLAN_PISTACHIO = 45
    COLORPLAN_POWDER_GREEN = 46
    COLORPLAN_PRISTINE_WHITE = 47
    COLORPLAN_PURPLE = 48
    COLORPLAN_RACING_GREEN = 49
    COLORPLAN_REAL_GREY = 50
    COLORPLAN_ROYAL_BLUE = 51
    COLORPLAN_SAPPHIRE = 52
    COLORPLAN_SCARLET = 53
    COLORPLAN_SMOKE = 54
    COLORPLAN_SORBET_YELLOW = 55  # Deprecated
    COLORPLAN_STONE = 56
    COLORPLAN_TABRIZ_BLUE = 57
    COLORPLAN_TURQUOISE = 58
    COLORPLAN_VELLUM_WHITE = 59
    COLORPLAN_VERMILLION = 60
    COLORPLAN_WHITE_FROST = 61

    def get_value(self) -> int:
        return int(self.value)



from __future__ import annotations

from typing import Protocol, runtime_checkable, Tuple, Any, TYPE_CHECKING

from pydantic import BaseModel

if TYPE_CHECKING:
    from mixam_sdk.item_specification.models import Embellishments, CustomSize
    from mixam_sdk.item_specification.models.foiling import Foiling
    from mixam_sdk.item_specification.models.substrate import Substrate
    from mixam_sdk.item_specification.enums.shape import Shape
    from mixam_sdk.item_specification.enums.component_type import ComponentType
    from mixam_sdk.item_specification.enums.standard_size import StandardSize
    from mixam_sdk.item_specification.enums.orientation import Orientation
    from mixam_sdk.item_specification.enums.colours import Colours
    from mixam_sdk.item_specification.enums.pre_drilled_holes import PreDrilledHoles
    from mixam_sdk.item_specification.enums.lamination import Lamination

class Member(BaseModel):
    code: str

class Container(BaseModel):
    code: str

def member_meta(code: str) -> dict[str, Member]:
    return {"member": Member(code=code)}

def container_meta(code: str) -> dict[str, Container]:
    return {"container": Container(code=code)}

@runtime_checkable
class Component(Protocol):
    format: int
    component_type: ComponentType
    standard_size: StandardSize
    custom_size: CustomSize | None
    orientation: Orientation
    substrate: Substrate
    colours: Colours
    pre_drilled_holes: PreDrilledHoles
    embellishments: Embellishments

    def has_back(self) -> bool: ...
    def is_folded(self) -> bool: ...
    def has_custom_size(self) -> bool: ...
    def is_laminated_component(self) -> bool: ...
    def is_two_sided_component(self) -> bool: ...
    def is_foiled_component(self) -> bool: ...
    def is_shaped_component(self) -> bool: ...

@runtime_checkable
class LaminatedComponent(Component, Protocol):
    lamination: Lamination

@runtime_checkable
class FoiledComponent(Component, Protocol):
    foiling: Foiling

@runtime_checkable
class TwoSidedComponent(LaminatedComponent, FoiledComponent, Protocol):
    back_colours: Colours
    back_lamination: Lamination
    back_foiling: Foiling

@runtime_checkable
class ShapedComponent(Component, Protocol):
    shape: Shape

SUBSTRATE_COMPARISON_FORMAT = "%s|%s|%s"

def _enum_key(x: Any) -> str:
    if x is None:
        return ""
    for attr in ("name", "value", "code"):
        if hasattr(x, attr):
            return str(getattr(x, attr))
    return str(x)


def component_comparator_key(c: Component) -> Tuple[str, str, str]:
    primary = _enum_key(getattr(c, "component_type", None))
    cls_name = c.__class__.__name__

    sub = c.substrate
    type_id = getattr(sub, "type_id", "")
    weight_id = getattr(sub, "weight_id", "")
    colour_id = getattr(sub, "colour_id", "")
    triplet = SUBSTRATE_COMPARISON_FORMAT % (type_id, weight_id, colour_id)

    return (primary, cls_name, triplet)


from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional

from mixam_sdk.item_specification.enums.component_type import ComponentType
from mixam_sdk.item_specification.enums.head_and_tail_bands import HeadAndTailBands
from mixam_sdk.item_specification.enums.pre_drilled_holes import PreDrilledHoles
from mixam_sdk.item_specification.enums.ribbon_colour import RibbonColour
from mixam_sdk.item_specification.models.bound_component import BoundComponent
from mixam_sdk.item_specification.models.cover_component import CoverComponent
from mixam_sdk.item_specification.models.item_specification import ItemSpecification
from mixam_sdk.metadata.product.enums.trilean import Trilean
from mixam_sdk.metadata.product.models.binding_type_option import BindingTypeOption
from mixam_sdk.metadata.product.models.bound_metadata import BoundMetadata
from mixam_sdk.metadata.product.models.substrate_type_metadata import SubstrateTypeMetadata
from mixam_sdk.metadata.product.services.validation_result import ValidationMessage


@dataclass(frozen=True)
class BindingTypeOptionMatch:
    binding_type_option: Optional[BindingTypeOption]
    errors: List[ValidationMessage]




def _validate_binding_type_option_against_item_spec(option: BindingTypeOption,
                                                    spec: ItemSpecification,
                                                    cover_substrate_types: List[SubstrateTypeMetadata]) -> List[ValidationMessage]:
    violations: List[ValidationMessage] = []
    bound: BoundComponent = spec.get_component(ComponentType.BOUND)

    # Sewing support
    if bound.binding.sewn and option.sewing == Trilean.UNAVAILABLE:
        violations.append(ValidationMessage(
            path="boundComponent.binding.sewn",
            message="Sewing is not available for this binding type",
            code="bound.sewing.unsupported",
        ))
    # Ribbons
    if not option.supports_ribbons and bound.ribbon_colour != RibbonColour.NONE:
        violations.append(ValidationMessage(
            path="boundComponent.ribbonColour",
            message="Ribbons are not supported by this binding type",
            code="bound.ribbon.unsupported",
        ))
    # Head & tail bands
    if not option.supports_head_and_tail_bands and bound.binding.head_and_tail_bands != HeadAndTailBands.NONE:
        violations.append(ValidationMessage(
            path="boundComponent.binding.headAndTailBands",
            message="Head & Tail Bands are not supported by this binding type",
            code="bound.head_and_tail_bands.unsupported",
        ))
    # Pre-drilled holes policy
    if option.pre_drilled_holes == Trilean.REQUIRED and bound.pre_drilled_holes == PreDrilledHoles.NONE:
        violations.append(ValidationMessage(
            path="boundComponent.preDrilledHoles",
            message="Pre-drilled holes are required for this binding type",
            code="bound.pre_drilled_holes.required",
        ))
    elif option.pre_drilled_holes == Trilean.UNAVAILABLE and bound.pre_drilled_holes != PreDrilledHoles.NONE:
        violations.append(ValidationMessage(
            path="boundComponent.preDrilledHoles",
            message="Pre-drilled holes are not available for this binding type",
            code="bound.pre_drilled_holes.unavailable",
        ))

    # Binding colour options
    if option.colour_options:
        has_colour = any(co.binding_colour == bound.binding.colour for co in option.colour_options)
        if not has_colour:
            violations.append(ValidationMessage(
                path="boundComponent.binding.colour",
                message="Unsupported Binding Colour for this binding type",
                code="bound.binding_colour.unavailable",
                extra={"allowed": [co.binding_colour.name for co in option.colour_options]},
            ))
    else:
        # Default/none behaviour: only BLACK is acceptable
        if bound.binding.colour.name != "BLACK":
            violations.append(ValidationMessage(
                path="boundComponent.binding.colour",
                message="Binding colour is not supported by this binding type",
                code="bound.binding_colour.unsupported",
            ))

    # Cover requirement availability
    if option.separate_cover == Trilean.REQUIRED and not spec.has_component(ComponentType.COVER):
        violations.append(ValidationMessage(
            path="coverComponent",
            message="A cover component is required for this product",
            code="bound.cover.required",
        ))
    elif option.separate_cover == Trilean.UNAVAILABLE and spec.has_component(ComponentType.COVER):
        violations.append(ValidationMessage(
            path="coverComponent",
            message="Covers are not supported by this product",
            code="bound.cover.unavailable",
        ))

    if spec.has_component(ComponentType.COVER):
        cover: CoverComponent = spec.get_component(ComponentType.COVER)
        is_valid_substrate = True
        if option.required_substrate is not None:
            is_valid_substrate = (
                option.required_substrate.type_id == cover.substrate.type_id and
                option.required_substrate.colour_id == cover.substrate.colour_id and
                option.required_substrate.weight_id == cover.substrate.weight_id
            )
        else:
            st = next((t for t in cover_substrate_types if t.id == cover.substrate.type_id), None)
            if st is None:
                is_valid_substrate = False
            else:
                sc = next((c for c in st.substrate_colours if c.id == cover.substrate.colour_id), None)
                if sc is None:
                    is_valid_substrate = False
                else:
                    sw = next((w for w in sc.weights if w.id == cover.substrate.weight_id), None)
                    is_valid_substrate = sw is not None
        if not is_valid_substrate:
            violations.append(ValidationMessage(
                path="coverComponent.substrate",
                message=(
                    f"Invalid substrate combination. Type ID: {cover.substrate.type_id}, "
                    f"Colour ID: {cover.substrate.colour_id}, Weight ID: {cover.substrate.weight_id}"
                ),
                code="cover.substrate.combo.invalid",
            ))

    from mixam_sdk.item_specification.enums.component_type import ComponentType as _CT
    if not option.supports_end_papers and spec.has_component(_CT.END_PAPERS):
        violations.append(ValidationMessage(
            path="endPaperComponent",
            message="End papers are not supported by this binding type",
            code="bound.end_papers.unsupported",
        ))
    if option.supports_end_papers and not spec.has_component(_CT.END_PAPERS):
        violations.append(ValidationMessage(
            path="endPaperComponent",
            message="EndPaper component is missing",
            code="bound.end_papers.missing",
        ))

    if not option.supports_dust_jacket and spec.has_component(_CT.DUST_JACKET):
        violations.append(ValidationMessage(
            path="dustJacketComponent",
            message="Dust Jackets are not supported by this binding type",
            code="bound.dust_jacket.unsupported",
        ))

    return violations


def select_binding_type_option_for_item_specification(spec: ItemSpecification,
                                                       cover_substrate_types: List[SubstrateTypeMetadata],
                                                       bound_metadata: BoundMetadata) -> BindingTypeOptionMatch:
    bound: BoundComponent = spec.get_component(ComponentType.BOUND)

    fewest_violations: List[ValidationMessage] | None = None
    selected: Optional[BindingTypeOption] = None

    for option in bound_metadata.binding_type_options:
        # Must match on binding type first
        if option.binding_type != bound.binding.type:
            continue
        violations = _validate_binding_type_option_against_item_spec(option, spec, cover_substrate_types)
        if not violations:
            selected = option
            fewest_violations = []
            break
        if fewest_violations is None or len(violations) < len(fewest_violations):
            fewest_violations = violations

    return BindingTypeOptionMatch(binding_type_option=selected, errors=fewest_violations or [])



from __future__ import annotations

from typing import ClassVar, Dict

from pydantic import Field, ConfigDict

from mixam_sdk.item_specification.interfaces.component_protocol import container_meta
from mixam_sdk.item_specification.models.foiling import Foiling
from mixam_sdk.item_specification.models.laminated_component_support import (
    LaminatedComponentSupport,
)


class LaminatedFoiledComponentSupport(LaminatedComponentSupport):

    FIELDS: ClassVar[Dict[str, str]] = {
        "foiling": "f",
    }

    foiling: Foiling = Field(
        default_factory=Foiling,
        description="Foiling details of the component.",
        json_schema_extra=container_meta(FIELDS["foiling"]),
    )

    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
        frozen=False,
        strict=True,
        validate_assignment=True
    )




from __future__ import annotations

from typing import Annotated, Optional

from pydantic import BaseModel, Field, ConfigDict

from mixam_sdk.item_specification.enums.shape import Shape
from mixam_sdk.item_specification.enums.sheet_size import SheetSize
from mixam_sdk.item_specification.enums.orientation import Orientation
from mixam_sdk.utils.enum_json import enum_by_name_or_value, enum_dump_name
from mixam_sdk.metadata.product.enums.trilean import Trilean


class ShapeOption(BaseModel):
    """
    Metadata model mirroring ShapeOption from Java services.
    Represents a single shape configuration option for a given size, including
    optional sheet information and rounded corners availability.
    """

    shape: Annotated[Shape, enum_by_name_or_value(Shape), enum_dump_name] = Field(
        alias="shape",
    )

    label: str = Field(
        alias="label",
    )

    value: int = Field(
        alias="value",
    )

    orientation: Annotated[
        Orientation, enum_by_name_or_value(Orientation), enum_dump_name
    ] = Field(
        alias="orientation",
    )

    amount_per_sheet: int = Field(
        default=1,
        alias="amountPerSheet",
    )

    sheet_size: Annotated[
        SheetSize, enum_by_name_or_value(SheetSize), enum_dump_name
    ] = Field(
        default=SheetSize.NOT_APPLICABLE,
        alias="sheetSize",
    )

    # Use a forward reference to avoid circular import issues with StandardSizeMetadata
    sheet_size_metadata: Optional["StandardSizeMetadata"] = Field(
        default=None,
        alias="sheetSizeMetadata",
    )

    rounded_corners: Annotated[
        Trilean, enum_by_name_or_value(Trilean), enum_dump_name
    ] = Field(
        default=Trilean.UNAVAILABLE,
        alias="roundedCorners",
    )

    santa_default: bool = Field(
        default=False,
        alias="santaDefault",
    )

    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",
        frozen=True,
        strict=True,
        validate_assignment=True,
    )


__all__ = ["ShapeOption"]



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Optional

from mixam_sdk.item_specification.models.value_based import ValueBased


class SubstrateWeightType(ValueBased, Enum):
    TEXT = 0
    COVER = 1

    def get_value(self) -> int:
        return int(self.value)


@dataclass(frozen=True)
class SubstrateWeight:
    value: int
    gsm: int
    caliper: float
    lbs: Optional[int] = None
    weight_type: Optional[SubstrateWeightType] = None



from mixam_sdk.utils.utils import (
    MM_IN_INCH,
    MM_IN_POINT,
    MM_IN_300_DPI_PIXEL,
    INCH_IN_POINT,
    INCH_IN_300_DPI_PIXEL,
    PAGE_SIZE_UNKNOWN,
    PAGE_SIZE_SPINE,
    PAGE_SIZE_JACKET,
    PAGE_SIZE_TOLERANCE,
    SECONDARY_SIZE_TOLERANCE,
    PAGE_HEIGHT_TOLERANCE,
    INACCURACY,
    DEFAULT_BLEED,
    DIN_SIZES,
    B_SIZES,
    SecondarySizeInfo,
    get_din_sizes,
    get_b_sizes,
    get_din_size_index,
    get_din_size_from_layout,
    get_din_dimensions,
    validate_secondary_size,
    validate_secondary_size_from_layout,
    get_secondary_name,
    get_secondary_size,
    is_dim_of_size,
    mm_to_inch,
    mm_to_point,
    mm_to_300dpi_pixel,
    inch_to_mm,
    inch_to_point,
    inch_to_300dpi_pixel,
    calculate_max_vertical_items_per_plate,
    calculate_max_horizontal_items_per_plate,
    get_b_format_from_custom_size,
)

__all__ = [
    # Constants
    "MM_IN_INCH",
    "MM_IN_POINT",
    "MM_IN_300_DPI_PIXEL",
    "INCH_IN_POINT",
    "INCH_IN_300_DPI_PIXEL",
    "PAGE_SIZE_UNKNOWN",
    "PAGE_SIZE_SPINE",
    "PAGE_SIZE_JACKET",
    "PAGE_SIZE_TOLERANCE",
    "SECONDARY_SIZE_TOLERANCE",
    "PAGE_HEIGHT_TOLERANCE",
    "INACCURACY",
    "DEFAULT_BLEED",

    # Data
    "DIN_SIZES",
    "B_SIZES",
    "SecondarySizeInfo",

    # Methods
    "get_din_sizes",
    "get_b_sizes",
    "get_din_size_index",
    "get_din_size_from_layout",
    "get_din_dimensions",
    "validate_secondary_size",
    "validate_secondary_size_from_layout",
    "get_secondary_name",
    "get_secondary_size",
    "is_dim_of_size",
    "mm_to_inch",
    "mm_to_point",
    "mm_to_300dpi_pixel",
    "inch_to_mm",
    "inch_to_point",
    "inch_to_300dpi_pixel",
    "calculate_max_vertical_items_per_plate",
    "calculate_max_horizontal_items_per_plate",
    "get_b_format_from_custom_size",
]
